/* Copyright 2011, CloudShopper (josue) - http://cloudshopper.com */
// start IIFE
(function(window, undefined){

    var DEV = 0;  // DEV MODE: 0 = Production, 1 = Development, 2 = Test (test.cloudshopper.com)

    var DOM = window.document||{}, NAV = window.navigator||{}, LOC = window.location||{};

    // if window is not parent then return window.
    if(window.top && window.top !== window) { return this; }

    var HTTP = DEV ? "http:" : (/https:/i.test(LOC.protocol) ? "https:" : "http:");
    var DOMAIN = HTTP+"//"+(DEV?"dev.":"")+"cloudshopper.com";
    var CDN = HTTP+"//cdn."+(DEV?"dev.":"")+"cloudshopper.com";
    var CDN_STATIC = (DEV ? CDN+"/static" : CDN);
    if(HTTP=="https:") {
        CDN = "https://cloudshopper-static.s3.amazonaws.com";
        CDN_STATIC = CDN;
    }
    
    var debug = function(){ console.log(arguments); },
        now = function(){ return parseInt((+new Date()).toString().substr(0,10),10); },
        e = function(e) { return DOM.getElementById(e); };

    var Storage = (function(){
        var A = {}, e = typeof simpleStorage === "object" && simpleStorage || window.SimpleStorage || window.localStorage;
        A.set = function(k,v) { v = typeof v == "object" && JSON.stringify(v) || v; e[k] = v; return this; };
        A.get = function(k) { return e[k] || null; };
        A.del = function(k) { if(e[k]) { delete e[k]; } return this; };
        A.getAll = function() { return e || null; };
        A.removeAll = function() { for(var k in e) { delete e[k]; } return this; };
        return A;
    })();

    var URI = {
        encode: function(s){ return encodeURIComponent(s||''); },
        build: function(o) {
            var q = [];
            for(var i in o) {
                var v = o[i] || "";
                v = /string/.test(typeof(v)) ? this.encode(v) : v;
                q.push(i+"="+v);
            }
            return q.join("&");
        }
    };
    var browser = (function(window){
        var name = 'unknown', version = 0;
        var ua = NAV && NAV.userAgent || '';
        if(/(chrome|firefox)\/[0-9\.]+/gim.test(ua)) {
            var browser = ua.match(/(chrome|firefox|safari)\/([0-9\.]+)/gim)[0].replace("/","-").toLowerCase().split('-');
            name = browser[0];
            version = browser[1];
        }
        else if(/(safari|opera)/gim.test(ua) && /version\/[0-9\.]+/gim.test(ua)) {
            name = ua.match(/(safari|opera)\/[0-9\.]+/gim)[0].split("/")[0].toLowerCase();
            version = ua.match(/version\/([0-9\.]+)/gim)[0].split("/")[1];
        }
        return { name:name, version:version, agent:name+"-"+version };
    })(window);

    var Ajax = function(opts, success, fail) {
        var opt = opts || {}, xhr = new XMLHttpRequest();
        var url = opt.url || opts;
        var cb = opt.success || opt.callback || success;
        if(!url) { return false; }
        if(opt.data) { url = url+"?"+URI.build(opt.data); }
        xhr.url = opt.url;
        xhr.data = opt.data;
        xhr.onreadystatechange = function(){
            if(xhr.readyState == 4 && xhr.status==200 && cb) {
                var response = xhr.responseText;
                if(typeof response==="string" && opt.json) {
                    try { response = JSON.parse(response); } catch(e) {}
                }
                cb.call(this, response, xhr);
            }
            else if(xhr.readyState == 4 && (xhr.status && xhr.status!=200) && (opt.fail || fail)) {
                (opt.fail || fail).call(this, xhr);
            }
        };
        xhr.open((opt.method||"GET"), url, true);
        xhr.send();
    };

    // API Methods....

    var EXT = {};
    EXT.dev = DEV
    EXT.debug = debug;
    EXT.Storage = Storage;
    EXT.URI = URI;
    EXT.browser = browser;
    EXT.ajax = Ajax;
    EXT.jQuery = window.CS_jQuery || window.jQuery;

    EXT.Settings = {
        get: function(callback){
            var settings = {};
            var fields  = "guid installed_at success_page version_now version_last heartbeat_hash heartbeat_last open_page_last ext_api_url";

            fields = fields.split(" ");
            for(var i in fields) {
                var f = fields[i];
                settings[f] = Storage.get(f);
            }

            // add to storage if missing...
            if(!settings.guid) {
                settings.guid = (function(){
                    var uid = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx".replace(/[x]/g, function(c) {
                        var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
                        return v.toString(16);
                    });
                    return uid;
                })();
                Storage.set("guid", settings.guid);
            }
            if(!settings.installed_at) {
                settings.installed_at = now();
                Storage.set("installed_at", settings.installed_at);
            }

            EXT._settings = settings || null;

            if(typeof callback=="function") {
                callback(settings);
            }
            
            return settings;
        },
        set: function(opts) {
            var opt = opts || {};
            var current = this.get();
            var update = opt.updates || {};
            var removals = opt.removals || {};

            // local updates
            if(opt.manifest || EXT._manifest) {
                EXT._manifest = opt.manifest || EXT._manifest;
                EXT._version = EXT.browser.agent+":"+(EXT._manifest.version||0);
                // save versions
                current.version_now = EXT._manifest.version;
                current.version_last = (current.version_last=="null" || !current.version_last) ? EXT._manifest.version : current.version_last;
                Storage.set("version_now",current.version_now);
                Storage.set("version_last",current.version_last);
            }

            // live updates
            if(update.open_page) {
                if(!current.open_page_last || (current.open_page_last && current.open_page_last!=update.open_page)) {
                    current.open_page = update.open_page;
                    Storage.set("open_page_last",current.open_page);

                    if(update.success_page && !current.success_page) {
                        var t = now();
                        Storage.set("success_page",t);
                        current.success_page = t;
                    }
                }
            }

            if(update.upgrade_version) {
                Storage.set("version_last",current.version_now);
            }
            
            if(update.heartbeat && update.heartbeat instanceof Object) {
                Storage.set("heartbeat_hash",update.heartbeat.hash);
                Storage.set("heartbeat_last",update.heartbeat.last);
                current.heartbeat_hash = update.heartbeat.hash;
                current.heartbeat_last = update.heartbeat.last;
            }
            if(update.ext_api_url) {
                Storage.set("ext_api_url",update.ext_api_url);
                current.ext_api_url = update.ext_api_url;
            }

            // removals
            for(var r in removals) {
                if(current[r]) { delete current[r]; }
                Storage.del(r);
            }

            // save all updates
            EXT._settings = current;
            return current;
        }
    };

    EXT.Heartbeat = function(opts, callback){
        var opt = opts || {};
        var callback = typeof opts==="function" && opts || callback;
        var settings = EXT._settings || EXT.Settings.get();

        var qs = {
            via: "ext-bar",
            _t: now(),
            reason: opt.reason || "normal",
            ua: browser.agent,
            guid: settings.guid,
            installed_at: settings.installed_at,
            version_now: settings.version_now || (EXT._manifest && EXT._manifest.version),
            version_last: settings.version_last,
            heartbeat_hash: settings.heartbeat_hash,
            heartbeat_last: settings.heartbeat_last,
            success_page: settings.success_page || 0,
            open_page_last: settings.open_page_last && URI.encode(settings.open_page_last),
            ext_api_url: settings.ext_api_url
        };

        // extra qs params
        if(opt.qs) { for(var i in opt.qs) { qs[i] = opt.qs[i]; } }

        var signal = DOMAIN+"/heartbeat?" + URI.build(qs);

        Ajax({url:signal, json:1},function(json){
            if(typeof callback==="function") {
                callback.call(this, json);
            }
        }, function(){
            Ajax(DOMAIN+"/track?type=ext-"+browser.name+"&v="+qs.version_now+"&error=heartbeat:no-connection");
        });
    };

    // config delay timers
    EXT.delay_heartbeat = 1800000; // every 30 minutes

    EXT.setExtSettings = function(opts){
        var opt = opts || {};
        var settings = opt.settings || {};
        var id = "cloudshopper_ext_settings";
        if(e(id)) { return false; }
        var ext = DOM.createElement("script");
        ext.id = id;
        ext.text  = "window.CS = window.CS||{}; CS._settings = \""+(opt.version||0)+":"+(settings.guid||0)+":"+(settings.installed_at||0)+":"+now()+"\"; ";
        if(opt.blocked_sites) {
            ext.text += "CS._blocked_sites = \""+settings.blocked_sites+"\"";
        }
        if(DEV) { ext.text += "CS._dev = 1; "; }
        DOM.body.appendChild(ext);
    };

    EXT.addFile = function(opts){
        var opt = opts || {};
        var settings = opt.settings || {};
        var id = opt.id || "cloudshopper_js_"+(+new Date);
        if(e(id)) {
            typeof opt.onload === "function" && opt.onload();
            return false;
        }
        //DEV && debug("addFile: "+id, opt.src, opt);
        var api = DOM.createElement("script");
        api.id = id;
        api.src = opt.src + (DEV?"?_t="+now():"");
        api.async = true;
        api.onload = opt.onload || false;
        DOM.body.appendChild(api);
    };
    EXT.setExtApi = function(){
        EXT.addFile({
            id: "cloudshopper_ext_essential",
            src: CDN_STATIC+"/essential"+(DEV?".debug":"")+".js"
        });
    };
    EXT.setExtSearch = function(callback){
        EXT.addFile({
            id: "cloudshopper_ext_essential",
            src: CDN_STATIC+"/essential"+(DEV?".debug":"")+".js",
            onload: function(){
                var ext = DOM.createElement("script");
                ext.id = "cloudshopper_ext_load_search";
                ext.text  = "window.CS && CS.api && CS.api({ via:'chromeicon', module:'modal', payload:{load:'search'} });";
                var head = DOM.getElementsByTagName("head").item(0);
                head.appendChild(ext);
                head.removeChild(ext);
            }
        });
    };

    var CS = window.CS || {};
    CS.Extension = CS.Extension || {};
    CS.Extension.BAR = EXT;
    window.CS = CS;

    window.debug = window.debug || debug;

    // is Ready
    DEV && debug("CS Extension is Ready - "+(DEV?"DEV":"")+" - "+EXT.browser.agent);
    
})(window);
// end IIFE
