var XUL_APP = require("xul-app");
var Request = require("request").Request;
var simpleStorage = require('simple-storage').storage;

/* Copyright 2011, CloudShopper (josue) - http://cloudshopper.com */
// start IIFE

// FIREFOX STARTUP ONLY
(function(exports){

	var DEV = 0;  // DEV MODE: 0 = Production, 1 = Development, 2 = Test (test.cloudshopper.com)

	var DOM = {}, NAV = {}, LOC = {};

	var HTTP = DEV ? "http:" : (/https:/i.test(LOC.protocol) ? "https:" : "http:");
    var DOMAIN = HTTP+"//"+(DEV?"dev.":"")+"cloudshopper.com";
    var CDN = HTTP+"//cdn."+(DEV?"dev.":"")+"cloudshopper.com";
    var CDN_STATIC = (DEV ? CDN+"/static" : CDN);
    if(HTTP=="https:") {
        CDN = "https://cloudshopper-static.s3.amazonaws.com";
        CDN_STATIC = CDN;
    }
	
	var debug = function(s){ console.log(JSON.stringify(s)); },
		now = function(){ return parseInt((+new Date()).toString().substr(0,10),10); },
		e = function(e) { return DOM.getElementById(e); };

	var Storage = (function(){
		var A = {}, e = simpleStorage;
		A.set = function(k,v) { v = typeof v == "object" && JSON.stringify(v) || v; e[k] = v; return this; };
		A.get = function(k) { return e[k] || null; };
		A.del = function(k) { if(e[k]) { delete e[k]; } return this; };
		A.getAll = function() { return e || null; };
		A.removeAll = function() { for(var k in e) { delete e[k]; } return this; };
		return A;
	})();

	var URI = {
		encode: function(s){ return encodeURIComponent(s||''); },
		build: function(o) {
			var q = [];
			for(var i in o) {
				var v = o[i] || "";
				v = /string/.test(typeof(v)) ? this.encode(v) : v;
				q.push(i+"="+v);
			}
			return q.join("&");
		}
	};

	var browser = { name:"firefox", version:XUL_APP.version||0 };
	browser.agent = browser.name+"-"+browser.version;

	var Ajax = function(opts, success, fail) {
		var opt = opts || {};
		var url = opt.url || opts;
		if(!url) { return false; }
		if(opt.data) { url = url+"?"+URI.build(opt.data); }
		Request({
			url: url,
			onComplete: function(data){
				var data = data.text;
				if(data && opt.json) {
					data = JSON.parse(data);
				}
				typeof success === "function" && success(data);
			}
		}).get();
	};

	// API Methods....

	var EXT = {};
	EXT.dev = DEV
	EXT.debug = debug;
	EXT.DOMAIN = DOMAIN;
	EXT.CDN = CDN_STATIC;
	EXT.Storage = Storage;
	EXT.URI = URI;
	EXT.browser = browser;
	EXT.ajax = Ajax;

	EXT.Settings = {
		get: function(callback){
			var settings = {};
			var fields  = "guid installed_at success_page version_now version_last heartbeat_hash heartbeat_last open_page_last ext_api_url";

			fields = fields.split(" ");
			for(var i in fields) {
				var f = fields[i];
				settings[f] = Storage.get(f);
			}

			// add to storage if missing...
			if(!settings.guid) {
				settings.guid = (function(){
					var uid = "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx".replace(/[x]/g, function(c) {
						var r = Math.random()*16|0, v = c == 'x' ? r : (r&0x3|0x8);
						return v.toString(16);
					});
					return uid;
				})();
				Storage.set("guid", settings.guid);
			}
			if(!settings.installed_at) {
				settings.installed_at = now();
				Storage.set("installed_at", settings.installed_at);
			}

			EXT._settings = settings || null;

			if(typeof callback=="function") {
				callback(settings);
			}
			
			return settings;
		},
		set: function(opts) {
			var opt = opts || {};
			var current = this.get();
			var update = opt.updates || {};
			var removals = opt.removals || {};

			// local updates
			if(opt.manifest || EXT._manifest) {
				EXT._manifest = opt.manifest || EXT._manifest;
				EXT._version = EXT.browser.agent+":"+(EXT._manifest.version||0);
				// save versions
				current.version_now = EXT._manifest.version;
				current.version_last = (current.version_last=="null" || !current.version_last) ? EXT._manifest.version : current.version_last;
				Storage.set("version_now",current.version_now);
				Storage.set("version_last",current.version_last);
			}

            // live updates
            if(update.open_page) {
                if(!current.open_page_last || (current.open_page_last && current.open_page_last!=update.open_page)) {
                    current.open_page = update.open_page;
                    Storage.set("open_page_last",current.open_page);

                    if(update.success_page && !current.success_page) {
                        var t = now();
                        Storage.set("success_page",t);
                        current.success_page = t;
                    }
                }
            }

            if(update.upgrade_version) {
                Storage.set("version_last",current.version_now);
            }

			if(update.heartbeat && update.heartbeat instanceof Object) {
				Storage.set("heartbeat_hash",update.heartbeat.hash);
				Storage.set("heartbeat_last",update.heartbeat.last);
				current.heartbeat_hash = update.heartbeat.hash;
				current.heartbeat_last = update.heartbeat.last;
			}
			if(update.ext_api_url) {
				Storage.set("ext_api_url",update.ext_api_url);
				current.ext_api_url = update.ext_api_url;
			}

			// removals
			for(var r in removals) {
				if(current[r]) { delete current[r]; }
				Storage.del(r);
			}

			// save all updates
			EXT._settings = current;
			return current;
		}
	};

	EXT.Heartbeat = function(opts, callback){
		var opt = opts || {};
		var callback = typeof opts==="function" && opts || callback;
		var settings = EXT._settings || EXT.Settings.get();

		var qs = {
			via: "ext-bar",
			_t: now(),
			reason: opt.reason || "normal",
			ua: browser.agent,
			guid: settings.guid,
			installed_at: settings.installed_at,
			version_now: settings.version_now || (EXT._manifest && EXT._manifest.version),
			version_last: settings.version_last,
			heartbeat_hash: settings.heartbeat_hash,
			heartbeat_last: settings.heartbeat_last,
			success_page: settings.success_page || 0,
			open_page_last: settings.open_page_last && URI.encode(settings.open_page_last),
			ext_api_url: settings.ext_api_url
		};

		// extra qs params
		if(opt.qs) { for(var i in opt.qs) { qs[i] = opt.qs[i]; } }

		var signal = DOMAIN+"/heartbeat?" + URI.build(qs);

		Ajax({url:signal, json:1},function(json){
			if(typeof callback==="function") {
				callback.call(this, json);
			}
		}, function(){
			Ajax(DOMAIN+"/track?error=ext-heartbeat:cannot_connect");
		});
	};

	// config delay timers
	EXT.delay_heartbeat = 300000; // every 5 minutes

	EXT.setExtSettings = function(opts){
		var opt = opts || {};
		var settings = opt.settings || {};
		var id = "cloudshopper_ext_settings";
		if(e(id)) { return false; }
		var ext = DOM.createElement("script");
		ext.id = id;
		ext.text  = "window.CS = window.CS||{}; CS.Extension=CS.Extension||{}; CS.Extension._settings = \""+(opt.version||0)+":"+(settings.guid||0)+":"+(settings.installed_at||0)+":"+now()+"\"; ";
		DOM.body.appendChild(ext);
	};

	EXT.setExtApi = function(opts){
		var opt = opts || {};
		var settings = opt.settings || {};
		var id = "cloudshopper_ext_api";
		if(e(id)) { return false; }
		var api = DOM.createElement("script");
		api.id = id;
		api.src = (settings.ext_api_url || CDN_STATIC + "/essential.debug.js") + (DEV?"?_t="+now():"");
		api.async = true;
		DOM.body.appendChild(api);
	};

    EXT.setExtSearch = function(callback){
	    var ext = document.createElement("script");
	    ext.id = "cloudshopper_ext_load_search";
	    ext.text  = "window.CS && CS.api && CS.api({ via:'chromeicon', module:'modal', payload:{load:'search'} });";
	    var head = document.getElementsByTagName("head").item(0);
	    head.appendChild(ext);
	    head.removeChild(ext);
    };

	// is Ready
	DEV && debug("CS Extension is Ready - "+(DEV?"DEV":"")+" - "+EXT.browser.agent);

	exports.CloudShopper = EXT;
	
})(exports);