var Chrome = require("chrome");
var timer = require("timer");
var pageMod = require("page-mod");
var tabs = require("tabs");
var simpleStorage = require('simple-storage').storage;
var toolbar = require("api-utils/toolbarbutton");

const widgets = require("widget");
const self = require("self");
var data = self.data;
var EXT = require("cs.firefox.js").CloudShopper;
var workers = [];
var button = null;

exports.main = function(options, callbacks) {

	var button_icon = {
		on: data.url('cs_ff_logo_16_active.png'),
		off: data.url('cs_ff_logo_16_inactive.png')
	};
	var button_options = {
	    id: "CloudShopper-Modal-Search",
	    label: "CloudShopper Search",
	    image: button_icon.off,
	    toolbarID: "nav-bar",
	    forceMove: true,
		onCommand: function(){
			for (var i = 0; i < workers.length; i++) {
				if (workers[i].tab == tabs.activeTab) {
					workers[i].port.emit("CS-EXT-DOM-LOAD","CS-EXT-MODAL-DHD");
				}
			}
		}
	};

	// install the CS Modal Search
	button = toolbar.ToolbarButton(button_options);
	button.moveTo({
		toolbarID: "nav-bar",
		forceMove: true
	});

	// get manifest and init settings
	EXT._settings = EXT.Settings.set({ manifest:self });

	// Firefox specific params/methods...
	EXT.firefox = {};
	EXT.firefox.backgroundScript = function() {
		// config local settings for contentScript
		var config = { settings: EXT._settings, version: EXT._version };
		var bg  = "if(unsafeWindow.self == unsafeWindow.top) { ";
			bg += "window.CS && CS.Extension && CS.Extension.BAR && CS.Extension.BAR.setExtSettings("+JSON.stringify(config)+"); ";
			bg += "window.CS && CS.Extension && CS.Extension.BAR && CS.Extension.BAR.setExtApi("+JSON.stringify(config)+"); ";
			bg += "self.port.on('CS-EXT-DOM-LOAD', function(name) { if(name=='CS-EXT-MODAL-DHD'){ window.CS && CS.Extension && CS.Extension.BAR && CS.Extension.BAR.setExtSearch(); } }); ";
			bg += "self.port.emit('CS-EXT-DOM-READY', { 'url':document.URL }); ";
			bg += "window.addEventListener('beforeunload', function(event) { self.port.emit('CS-EXT-DOM-LEAVE', { 'url':document.URL }); }, false); ";
			bg += "} ";

		return bg;
	};

	EXT._heartbeat = function(opts, callback){
		EXT.Heartbeat(opts, function(response){
			EXT._settings = EXT.Settings.set(response);
			EXT.firefox.contentScript = EXT.firefox.backgroundScript();
			if(EXT._settings.open_page) {
				tabs.open(EXT._settings.open_page);
			}
			EXT.dev && EXT.debug("Heartbeat Status: " + (response.log||"OK"));
			if(typeof callback==="function") {
				callback.call(this);
			}
		});
	};

	// start checking heartbeat
	EXT._heartbeat({ reason:options.loadReason }, function(){
		// dev timers then start timed check.
		EXT.delay_heartbeat = EXT.dev ? 30000 : EXT.delay_heartbeat;
		EXT._checking_heartbeat = timer.setInterval(EXT._heartbeat, EXT.delay_heartbeat);

		// contentScript for all pages...
		pageMod.PageMod({
			include: ['*'],
			contentScriptWhen: 'ready',
			contentScriptFile: [data.url('cs.js')],
			contentScript: EXT.firefox.contentScript,
			onAttach: function(worker) {
				workers.push(worker);
				worker.port.on('CS-EXT-DOM-READY', function(message) {
					if(worker.url.match(/^http/)) {
						button.setIcon({ url: button_icon.on });
					}
				});
				worker.port.on('CS-EXT-DOM-LEAVE', function(message) {
					if(worker.url.match(/^http/)) {
						button.setIcon({ url: button_icon.off });
					}
				});
				worker.on("detach", function(detached) {
					var index = workers.indexOf(worker);
					if (index >= 0) {
						workers.splice(index, 1);
					}
				});
			}
		});
	});

};

exports.onUnload = function(reason) {
	var ref_id = EXT._settings.guid || +(new Date);
	if(reason=="disable" || reason=="uninstall") {
		tabs.open(EXT.DOMAIN+"/bar/uninstalled?r="+reason+"&vn="+EXT._settings.version_now+"vl="+EXT._settings.version_last+"&b="+EXT.browser.agent+"&ia="+EXT._settings.installed_at+"&ref="+ref_id);
	}
	if(reason=="shutdown") {
		if(button && button.destroy) {
			button.destroy();
			button = null;
		}
	}
};